/**
   蓝牙模式，实现蓝牙键盘功能
   可通过蓝牙连接电脑和手机
 * **/
#include "keyboard.h"
#include "scan.h"
#include "save.h"
#include "BT.h"
#include "sleep.h"
//BlueTooth
#include "esp_bt_main.h"
#include "esp_bt.h"
#include "esp_sleep.h"
#include "esp_bt_device.h"
#include "BLEDevice.h"
#include <BleKeyboard.h>
//设置蓝牙键盘信息
//第一个参数为设备名称，第二个参数为设备制造商名称，第三个参数为电池点亮。
BleKeyboard bleKeyboard(DeviceName, Manufacturer, DevicePower);

#define BT_TASK_CORE 0     //CPU1
#define BT_TASK_PRIORITY 2 //优先级为2
#define BT_TASK_STACK 4096 //堆栈字数 堆栈位宽32
#define BT_TASK_DELAY 4    //任务延时4ms

TaskHandle_t BT_TASK_Handle = NULL;
char CAP_LED_Flag = 1;

//Set BT name
void getDeviceName(char* deviceName, char* devicePrefix)
{
  uint64_t chipid = ESP.getEfuseMac();
  uint16_t shortid = chipid >> 32;
  char sprintfString[20] = "";
  strcat(sprintfString, devicePrefix);
  strcat(sprintfString, " %04X");
  sprintf(deviceName, sprintfString, shortid);
}

/***********************************bt_task*************************************
   name: bt_task

   传参：

   返回值：

   功能：
   蓝牙任务
*******************************************************************************/
void bt_task(void *pvParameters)
{
  char deviceName[20];
  uint8_t keyboard_old[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
  esp_bt_controller_config_t bt_config = BT_CONTROLLER_INIT_CONFIG_DEFAULT();
  if (esp_bt_controller_init(&bt_config) != ESP_OK) {
    Serial.println("Failed to initialize Bluetooth controller");
    return;
  }

  if (esp_bt_controller_enable(ESP_BT_MODE_BLE) != ESP_OK) {
    Serial.println("Failed to enable Bluetooth controller");
    return;
  }

  if (esp_bluedroid_init() != ESP_OK) {
    Serial.println("Failed to initialize Bluetooth stack");
    return;
  }

  if (esp_bluedroid_enable() != ESP_OK) {
    Serial.println("Failed to enable Bluetooth stack");
    return;
  }

//  getDeviceName(deviceName, "ElecrowBT_");
//  bleKeyboard.setName(deviceName);
  bleKeyboard.begin();
  delay(1000);
  for (;;)
  {
    if (millis() - bt_connect_interval >= BT_CONNECT_TIME && BT_CONNECT_TIME > 0) //关机
    {
      Serial.println("键盘未连接进入休眠");
      board_sleep();
    }
    uint8_t keyboard_new[KEYBOARD_ROW][KEYBOARD_COLUMN] = {};
    if (bleKeyboard.isConnected())
    {
      bt_connect_interval = millis();
      if (bleKeyboard.getLedStatus(LED_CAPS_LOCK))
      {
        Wire.beginTransmission(0x2D); // Address of the STC8G
        Wire.write(0x01); // Value to write
        Wire.endTransmission();
      }
      else
      {
        Wire.beginTransmission(0x2D); // Address of the STC8G
        Wire.write(0x02); // Value to write
        Wire.endTransmission();
      }
    }
    if (xQueuePeek(KEYBOARD_SATE_QUEUE, &keyboard_new, 10) == pdTRUE)
    {
      if (keyboard_new[Fn_X][Fn_Y] == 1) //判断Fn按下
      {
        for (uint8_t i = 0; i < KEYBOARD_ROW; i++)
        {
          for (uint8_t j = 0; j < KEYBOARD_COLUMN; j++)
          {
            if (keyboard_new[i][j] != keyboard_old[i][j])
            {
              keyboard_old[i][j] = keyboard_new[i][j];
              if (keyboard_new[i][j] == 1)
              {
                bt_fn(1, i, j);
              }
              else if (keyboard_new[i][j] == 0)
              {
                bt_fn(0, i, j);
              }
              bt_press_interval = millis();
            }
          }
        }
      }
      else
      {
        for (uint8_t i = 0; i < KEYBOARD_ROW; i++)
        {
          for (uint8_t j = 0; j < KEYBOARD_COLUMN; j++)
          {
            if (keyboard_new[i][j] != keyboard_old[i][j])
            {
              keyboard_old[i][j] = keyboard_new[i][j];
              if (keyboard_new[i][j] == 1)
              {
                if (BT_KEY_CODE[i][j] != 0)
                {
                  if (bleKeyboard.isConnected())
                  {
#ifdef DEBUG_K
                    Serial.print("i: ");
                    Serial.print( i );
                    Serial.print( "\n" );
                    Serial.print("j: ");
                    Serial.print( j );
                    Serial.print( "\n" );
#endif
                    if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
                    {
                      bleKeyboard.press(BT_KEY_CODE[i][j]);
                    }
                    else
                    {
                      bleKeyboard.press(BT_KEY_MAC_CODE[i][j]);
                    }
                  }
                }
                if (i == CAPS_X && j == CAPS_Y) //大写锁定
                {

                  if (bleKeyboard.getLedStatus(LED_CAPS_LOCK))
                  {
                    Wire.beginTransmission(0x2D); // Address of the STC8G
                    Wire.write(0x01); // Value to write
                    Wire.endTransmission();
                  }
                  else
                  {
                    Wire.beginTransmission(0x2D); // Address of the STC8G
                    Wire.write(0x02); // Value to write
                    Wire.endTransmission();
                  }
                }
              }
              else if (keyboard_new[i][j] == 0)
              {
                if (BT_KEY_CODE[i][j] != 0)
                {
                  if (bleKeyboard.isConnected())
                  {
                    if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
                    {
                      bleKeyboard.release(BT_KEY_CODE[i][j]);
                    }
                    else
                    {
                      bleKeyboard.release(BT_KEY_MAC_CODE[i][j]);
                    }
                  }
                }
              }
              bt_press_interval = millis();
            }
          }
        }
      }
      if (digitalRead(KEY_ENTER_PIN))
      {
        KEY_ENTER_State = 1;
        bleKeyboard.press(KEY_RETURN);
      }
      else
      {
        if (KEY_ENTER_State == 1)
        {
          bleKeyboard.release(KEY_RETURN);
          KEY_ENTER_State = 0;
        }
      }
      if (millis() - bt_press_interval >= BT_PRESS_TIME && BT_PRESS_TIME > 0) //关机
      {
        board_sleep();
      }
    }
#ifdef BT_TASK_DELAY
    vTaskDelay(BT_TASK_DELAY);
#endif
  }
}

/*
   name: bt_task_begin

   返回值：
   蓝牙任务状态 0/1
   功能：
   开启蓝牙任务
 * */
uint8_t bt_task_begin()
{
  if (BT_TASK_Handle != NULL)
  {
    return 1;
  }
  else
  {
    BaseType_t xReturned;
    xReturned = xTaskCreatePinnedToCore(bt_task, "bt_task", BT_TASK_STACK, NULL, BT_TASK_PRIORITY, &BT_TASK_Handle, BT_TASK_CORE);
    if (xReturned == pdPASS)
    {
      return 1;
    }
  }
  return 0;
}

/*
   name: bt_task_end

   传参：
   返回值：
   蓝牙任务状态 0/1
   功能：
   关闭蓝牙任务
 * */
void bt_task_end()
{
  if (BT_TASK_Handle != NULL)
  {
    //关闭任务
    vTaskDelete(BT_TASK_Handle);
    BT_TASK_Handle = NULL;
    //判断蓝牙资源释放初始化
    if (esp_bluedroid_get_status() != ESP_BLUEDROID_STATUS_UNINITIALIZED)
    {
      esp_bluedroid_disable(); //关闭蓝牙
      esp_bluedroid_deinit();  //释放蓝牙资源
    }
    //判断蓝牙控制器是否开启
    if (esp_bt_controller_get_status() != ESP_BT_CONTROLLER_STATUS_IDLE)
    {
      esp_bt_controller_disable(); //关闭蓝牙控制器
      esp_bt_controller_deinit();  //释放蓝牙控制器资源
    }
  }
}

/*
   name: bt_fn

   传参：
   state_ : 按键状态
   i : 按键所在行
   j : 按键所在列

   功能：
   判断行和列
   按下Fn键后调用此函数进行按键处理

 * */
void bt_fn(uint8_t state_, uint8_t i, uint8_t j)
{
  if (i <= 0)
  {
    if (bleKeyboard.isConnected())
    {
      if (state_ == 1)
      {
        if (i == Fn_F1_X && j == Fn_F1_Y)
        {
          bleKeyboard.press(KEY_MEDIA_BRIGHTNESS_DOWN);
        }
        else if (i == Fn_F2_X && j == Fn_F2_Y)
        {
          bleKeyboard.press(KEY_MEDIA_BRIGHTNESS_UP);
        }
        else if (i == Fn_F3_X && j == Fn_F3_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_CALCULATOR);
          }
          else
          {

          }
        }
        else if (i == Fn_F4_X && j == Fn_F4_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_LOCAL_MACHINE_BROWSER);
          }
          else
          {

          }
        }
        else if (i == Fn_F5_X && j == Fn_F5_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_CONSUMER_CONTROL_CONFIGURATION);
          }
          else
          {

          }
        }
        else if (i == Fn_F6_X && j == Fn_F6_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_PREVIOUS_TRACK);
          }
          else
          {

          }
        }
        else if (i == Fn_F7_X && j == Fn_F7_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_NEXT_TRACK);
          }
          else
          {
            bleKeyboard.press(KEY_MEDIA_PREVIOUS_TRACK);
          }
        }
        else if (i == Fn_F8_X && j == Fn_F8_Y)
        {
          bleKeyboard.press(KEY_MEDIA_PLAY_PAUSE);
        }
        else if (i == Fn_F9_X && j == Fn_F9_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.press(KEY_MEDIA_STOP);
          }
          else
          {
            bleKeyboard.press(KEY_MEDIA_NEXT_TRACK);
          }
        }
        else if (i == Fn_F10_X && j == Fn_F10_Y)
        {
          bleKeyboard.press(KEY_MEDIA_MUTE);
        }
        else if (i == Fn_F11_X && j == Fn_F11_Y)
        {
          bleKeyboard.press(KEY_MEDIA_VOLUME_DOWN);
        }
        else if (i == Fn_F12_X && j == Fn_F12_Y)
        {
          bleKeyboard.press(KEY_MEDIA_VOLUME_UP);
        }
      }
      else
      {
        if (i == Fn_F1_X && j == Fn_F1_Y)
        {
          bleKeyboard.release(KEY_MEDIA_BRIGHTNESS_DOWN);
        }
        else if (i == Fn_F2_X && j == Fn_F2_Y)
        {
          bleKeyboard.release(KEY_MEDIA_BRIGHTNESS_UP);
        }
        else if (i == Fn_F3_X && j == Fn_F3_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_CALCULATOR);
          }
          else
          {
            bleKeyboard.press(KEY_LEFT_CTRL);
            bleKeyboard.press(KEY_UP_ARROW);
            bleKeyboard.releaseAll();
          }
        }
        else if (i == Fn_F4_X && j == Fn_F4_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_LOCAL_MACHINE_BROWSER);
          }
          else
          {
            bleKeyboard.press(KEY_F4);
            bleKeyboard.releaseAll();
          }
        }
        else if (i == Fn_F5_X && j == Fn_F5_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_CONSUMER_CONTROL_CONFIGURATION);
          }
          else
          {

          }
        }
        else if (i == Fn_F6_X && j == Fn_F6_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_PREVIOUS_TRACK);
          }
          else
          {

          }
        }
        else if (i == Fn_F7_X && j == Fn_F7_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_NEXT_TRACK);
          }
          else
          {
            bleKeyboard.release(KEY_MEDIA_PREVIOUS_TRACK);
          }
        }
        else if (i == Fn_F8_X && j == Fn_F8_Y)
        {
          bleKeyboard.release(KEY_MEDIA_PLAY_PAUSE);
        }
        else if (i == Fn_F9_X && j == Fn_F9_Y)
        {
          if (Keyboard_Config.Switch_system == WINDOWS_SYSTEM)
          {
            bleKeyboard.release(KEY_MEDIA_STOP);
          }
          else
          {
            bleKeyboard.release(KEY_MEDIA_NEXT_TRACK);
          }
        }
        else if (i == Fn_F10_X && j == Fn_F10_Y)
        {
          bleKeyboard.release(KEY_MEDIA_MUTE);
        }
        else if (i == Fn_F11_X && j == Fn_F11_Y)
        {
          bleKeyboard.release(KEY_MEDIA_VOLUME_DOWN);
        }
        else if (i == Fn_F12_X && j == Fn_F12_Y)
        {
          bleKeyboard.release(KEY_MEDIA_VOLUME_UP);
        }
        else if (i == Fn_ESC_X && j == Fn_ESC_Y) //切换复位灯光
        {
#ifdef DEBUG_K
          Serial.println("BT复位灯光");
#endif
          Keyboard_Config.L_mode = 0;
          set_light_mode(Keyboard_Config.L_mode);
          save_config();
        }
      }
    }
  }
  else
  {
    fn_set(state_, i, j);
  }
}
